// ---------------------------------------------------------------------------
//
#include "Matrices.hpp"

#define P(i,j)		P[(i)*(r) + (j)]
#define A(i,j)		A[(i)*(r+1) + (j)]
#define H(i,j)		H[(i)*(r+1) + (j)]
#define V(i,j)		V[(i)*(r+1) + (j)]

// ---------------------------------------------------------------------------
// Structure : Des squences
class CSeqs
{
	private:
		// Variables
		int 		TASK; 														// Version of ALIGNER
		int 		NGrps; 														// Number of groups
		int 		Doubled; 													// Duplicated sequences
		int 		SubMatrix; 													// Substitution matrix
		int 		OutFormat; 													// Input alignment format
		int 		GuideTree; 													// One tree for all sequences or one tree per cluster
		int 		Doubles; 													// If Keep the doubled sequences
		int 		AlignMode; 													// Perform global alignment
		int			LocalAlig;													// Perform local alignment in the detection of significant patterns
		int 		Refinment;													// Perform iteraive refinment of the alignments
		int         SaveGrps;													// If save groups of sequences
		bool 		SaveTrs; 													// If save phylogenetic trees
		bool 		SaveCls; 													// If save clustering result
		bool 		SaveAln; 													// If save alignment
		bool 		SaveMot; 													// If save motifs
		int 		Leaves; 													// Use IDs or names for Leaves
		int*		NSeqs; 														// Number of sequences on each group

		int**		Grps; 														// Sequences in each group
		float**		Matrix; 													// Distance matrix
		CMot*		Motifs; 													// Significant motifs
		CNode*		Root; 														// Root of the phylogenetic tree
		time_t		Start; 														// Starting time of execution

		// Phylogenetic tree
		void 		UpdtSubMatrix(); 											// Choose and update the substitution matrix
		void 		PhyAllTree(CNode *Node);									// Save global phylogenetic tree in Phylip file format
		void 		PhySubTree(CNode *Node, int Group); 						// Save phylogenetic subtree in Phylip file format
		void 		SaveTree();			 										// Save the phylogenetic tree
		void		SaveGroup(CNode *N);										// Save sequence of group

		// Clustering
		void 		MergeNodes(CNode *Node); 									// Regrouper les noeuds dans le groupe en cours
		void 		GetGroups(CNode *Node, float ThS); 							// Regrouper les noeuds par scores
		void 		ClusterNodes(CNode **Nodes); 								// Regrouper les noeuds par scores
		void 		GetClusters(); 												// Extract clusters
		void 		SaveClusters(); 											// Save Clusters
		void 		Hierarchical(); 											// Hierarchical clustering
		void 		Spectral1(); 												// Spectral decomposition clustering
		void 		Spectral2(); 												// Spectral decomposition clustering
		void 		SpanningTree(CList *D, CVertx *V, int Var, int Nbr);		// Build a spanning tree from a similarity matrix

		// Similarity measure
		int 		Karlin(CSeq *S1, CSeq *S2); 								// Expected length of the longest common motif (Karlin)
		void 		ExactMatch(CSeq *S1, CSeq *S2, int Min); 					// SMS matching score
		void 		PermtMatch(CSeq *S1, CSeq *S2, int Min); 					// SMS matching score

		// Alignment
		bool 		Significant(CNode *N, int Seq, int Res);					// Detect and collect significant patterns
		bool        Refinement(CNode *P, CNode *N);								// Alignment refinement
		void 		SaveToRTF(CNode *N); 	 									// Save alignment in rtf file format
		void 		SaveToTXT(CNode *N); 	 									// Save alignment in txt file format
		void 		AlignUpdt(CNode *N1, CNode *N2); 							// Epdate the alignment with the new one
		void 		InsertGap(CNode *N, int Pos, int Len);						// Insert object in a string
		void        CleanNode(CNode *N);										// Clean sequence in a node from gaped cloumns
		void 		Alignment(float *P, CNode *N); 								// Needleman and Wunsch global alignment algorithm
		void        SPatterns(CSeq *S1, CSeq *S2); 								// Build the profile matrix to align a pair of groups
		void 		AlignNode(CNode *N); 										// Align the two profiles
		bool 		AlignImpr(CNode *P, CNode *N);								// Alignment improvment
		void 		AlignGrps(CNode *N); 										// Alignment one subgroups
		void 		AlignSeqs(CNode *N); 										// Regrouper les squences
		float 		GlbPWAlig(float *P, CSeq *S1, CSeq *S2); 					// Alignment score of a pair of profiles
		float 		LocPWAlig(CSeq *S1, CSeq *S2); 								// Alignment score of a pair of profiles
		void        WeightSeq(CNode *N, float W);								// Weights of sequences
		void 		MatrixALIGNER(); 											// Pairwise similarity matrix

	public:
		// Creator Destructor
		CSeqs(int Default); // Constructor
		~CSeqs(); // Destructor

		// Sequences
		int 		Seqs; 														// Sequence number
		int 		SeqMax;														// Sequence index of maximum length
		CSeq*		Seq; 														// Length of the sequence

		// Default files
		char		InpFile[MAXLEN]; 											// Input file name
		char		OutFile[MAXLEN]; 											// Output file name
		char		PhyFile[MAXLEN]; 											// Output file name

		// Set input parameters
		void 		ALIGNEROptions(int argc, char **argv);						// Setup ALIGNER input parameters

		// Set ALIGNER environmenet options
		void 		FindDoubled(); 												// Remove doubles sequences
		void 		LoadFasta(); 												// Load sequences from FASTA format
		void 		Significant(); 												// Collect significant patterns
		void 		Similarity(); 												// Regrouper les squences
		void 		Clustering(); 												// Regrouper les squences
		void 		Alignment(); 												// Multiple alignment
};
// ---------------------------------------------------------------------------
// Constructor
CSeqs::CSeqs(int Default)
{
	// Varibales
	Seqs	= 0;
	Doubled = 0;
	SeqMax 	= 0;
	NGrps	= 0;
	Seq 	= NULL;
	NSeqs 	= NULL;
	Grps 	= NULL;
	Matrix 	= NULL;
	Root 	= NULL;
	Matrix 	= NULL;
	Motifs 	= NULL;

	// Set the default options
	TASK 	  = Default;
	SubMatrix = IP_BLOSUM62;
	OutFormat = IP_CLUSTAL;
	Doubles   = IP_DOUBLEO;
	AlignMode = IP_ALIGNSUB;
	LocalAlig = IP_LOCALALY;
	Refinment = IP_REFWI;
	Leaves 	  = IP_LEAFN;
	SaveGrps  = false;
	SaveTrs   = true;
	SaveCls   = true;
	SaveAln   = true;
	SaveMot   = true;

	// Set the default files
	sprintf_s(InpFile, MAXLEN, "File.fas");
	sprintf_s(OutFile, MAXLEN, "File.cls");
	sprintf_s(PhyFile, MAXLEN, "File.phy");
}
// ---------------------------------------------------------------------------
// Destructor
CSeqs::~CSeqs()
{
	for (int i=0; i<NGrps; i++)
	delete[] Grps[i];
	delete[] Grps;
	delete[] NSeqs;

	for (int i=0; i<Seqs; i++)
	{
		delete[] Seq[i].Res;

		if (Matrix != NULL) delete[] Matrix[i];
		if (Motifs != NULL) delete[] Motifs[i].Res;
	}

	delete[] Seq;
	if (Matrix != NULL) delete[] Matrix;
	if (Motifs != NULL) delete[] Motifs;
}
// ---------------------------------------------------------------------------
// Chose and uodate the substitution matrix
void CSeqs::UpdtSubMatrix()
{
	switch(SubMatrix)
	{
		case IP_BLOSUM62:	for (int i=0; i<SYMBOLS+1; i++)
							for (int j=0; j<SYMBOLS+1; j++)
							{
								SUB_MAT1[i][j] = BLOSUM62[i][j];
								SUB_MAT2[i][j] = BLOSUM62[i][j] + GapEX * 2.0f;
							}
							break;

		case IP_PAM250:		for (int i=0; i<SYMBOLS+1; i++)
							for (int j=0; j<SYMBOLS+1; j++)
							{
								SUB_MAT1[i][j] = PAM250[i][j];
								SUB_MAT2[i][j] = PAM250[i][j] + GapEX * 2.0f;
							}
							break;
		default:
		{
			SendMsg(FATALERR, "018");
			exit(EXIT_FAILURE);
		}
	}
}
// ---------------------------------------------------------------------------
// Set environmenet options
void CSeqs::ALIGNEROptions(int argc, char **argv)
{
	int  i, j;
	int  Mat;
	int  arg = 0;
	char Input[MAXLEN];

	if (argc == 1) // Intereactive mode
	{
		// Welocoming message
		printf("ALIGNER for detecting and aligning groups of related protein sequences\n");
		printf("ALIGNER is developed and implemented by Abdellali Kelil PhD.\n");
		printf("ALIGNER is a free softwar, you can redistribute it and/or modify it\n");
		printf("ALIGNER webserver is available at http://prospectus.usherbrooke.ca/ALIGNER/\n\n");
		printf("for help input : ALIGNER -hp\n\n");
		printf("==========\n\n");

		// Input file (fasta format)
		printf("Input filename : ");
		scanf("%s", InpFile);

		// Separation
		printf("\n");

		// Clustering algorithm
		printf("1: ALIGNER with strict matching, min length = 4\n");
		printf("2: ALIGNER with strict matching, min length online\n");
		printf("3: ALIGNER with permissive matching, min length online\n\n");
		printf("Choose ALIGNER version : ");
		scanf("%s", Input);
		switch(atoi(Input))
		{
			case 1:	TASK = ALIGNER1; break;
			case 2:	TASK = ALIGNER2; break;
			case 3: TASK = ALIGNER3; break;
			default:
			{
				SendMsg("Please, enter a valid number !!!", "027");
				exit(EXIT_FAILURE);
			}
		}

		// Separation
		printf("\n");

		// Penalty for opening gap
		printf("Open gap panalty (positive value) : ");
		scanf("%f", &GapOP);

		// Penalty for extension gap
		printf("Extension gap panalty (positive value) : ");
		scanf("%f", &GapEX);

		// Minimum score allowed for matching
		printf("Minimum score allowed for matcing : ");
		scanf("%f", &SbScr);

		// Separation
		printf("\n");

		// Substitution matrix
		printf(" 1: BLOSUM62\t2, PAM250\n");
		printf("Matrix choice: ");
		scanf("%s", Input);
		switch(atoi(Input))
		{
			case 1:	SubMatrix = IP_BLOSUM62;
					UpdtSubMatrix();
					break;

			case 2:	SubMatrix = IP_PAM250;
					UpdtSubMatrix();
					break;
			default:
			{
				SendMsg("Please, enter a valid number !!!", "028");
				exit(EXIT_FAILURE);
			}
		}

		// Separation
		printf("\n");

		// Redundant sequences
		printf("Withdraw redundant sequences (1:Yes, 2:No) : ");
		scanf("%s", Input);
		switch(atoi(Input))
		{
			case 1:	Doubles = IP_DOUBLEI; break;
			case 2: Doubles = IP_DOUBLEO; break;
			default:
			{
				SendMsg("Please, enter a valid number !!!", "029");
				exit(EXIT_FAILURE);
			}
		}

		// Separation
		printf("\n");

		// Save group of sequences
		printf("Save each cluster in a separate file (1:Yes, 2:No) : ");
		scanf("%s", Input);
		switch(atoi(Input))
		{
			case 1:	SaveGrps = true; break;
			case 2: SaveGrps = false; break;
			default:
			{
				SendMsg("Please, enter a valid number !!!", "029");
				exit(EXIT_FAILURE);
			}
		}

		// Separation
		printf("\n");
	}
	else // Argument mode
	{
		while (arg < argc)
		{
			// Input filename
			if (stricmp(argv[arg], "-hp") == 0)
			{
				printf("ALIGNER for detecting and aligning groups of related protein sequences\n");
				printf("ALIGNER is developed and implemented by Abdellali Kelil\n");
				printf("ALIGNER is a free softwar, you can redistribute it and/or modify it\n\n");
				printf("ALIGNER webserver is available at http://prospectus.usherbrooke.ca/ALIGNER/\n\n");
				printf("Usage1 : ALIGNER (follow the instructions)\n\n");
				printf("Usage2 : ALIGNER [OPTION1 ARGUMENT1 OPTION2 ARGUMENT2 ...]\n\n");
				printf("OPTIONS and ARGUMENTS\n");
				printf("=============================\n\n");
				printf("Input file:  -in [input file]\n\n");
				printf("Output file: -ou [output file]\n\n");
				printf("ALIGNER version: -vr [1, 2, 3, or 4] (default 3)\n\n");
				printf("\tversion 1 : detect patterns, strict matching, length = 4\n");
				printf("\tversion 2 : detect patterns, strict matching, length online\n");
				printf("\tversion 3 : detect patterns, permissive matching, length online\n");
				printf("\tversion 4 : do not detect patterns\n");
				printf("\tversion 5 : detect patterns without performing alignment\n\n");
				printf("Substitution matrix:             -sm [1:BLOSUM or 2:PAM250] (default 1)\n\n");
				printf("Opening gap penalty:             -op [positive value] (default 13.0)\n\n");
				printf("Extension gap penalty:           -ex [positive value] (default  2.0)\n\n");
				printf("Minimum score allowed:           -ms [positive value] (default  0.0)\n\n");
				printf("Keep or no redundent sequences:  -kd [1:keep or 2:withdraw] (default 2)\n\n");
				printf("Output alignment format:         -of [1:CLUSTAL, 2:FASTA, or 3:NATIVE] (default 1)\n\n");
				printf("Save or no phylogenetic tree:    -st [1:yes or 2:no] (default 1)\n\n");
				printf("Save or no clustering result:    -sc [1:yes or 2:no] (default 1)\n\n");
				printf("Save or no alignment result:     -sa [1:yes or 2:no] (default 1)\n\n");
				printf("Save or no signifiant motifs:    -sm [1:yes or 2:no] (default 1)\n\n");
				printf("Save or no clusters:             -sg [1:yes or 2:no] (default 1)\n\n");
				printf("Perform iterative refinment:     -ir [1:yes or 2:no] (default 1)\n\n");
				printf("Label leaves with Names or IDs : -tl [1:Names or 2:IDs] (default 1)\n\n");

				// Exit ALIGNER
				exit(0);
			}

			// Input filename
			if (stricmp(argv[arg], "-in") == 0) strcpy(InpFile, argv[arg+1]);

			// Output filename
			if (stricmp(argv[arg], "-ou") == 0) strcpy(OutFile, argv[arg+1]);

			// Alignment approach: ALIGNER1, ALIGNER2, ALIGNER3
			if (stricmp(argv[arg], "-vr") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1:	TASK = ALIGNER1; break;
					case 2:	TASK = ALIGNER2; break;
					case 3: TASK = ALIGNER3; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "030");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Substitution matrix: BLOSUM62, PAM250
			if (stricmp(argv[arg], "-ma") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1:	SubMatrix = IP_BLOSUM62; break;
					case 2:	SubMatrix = IP_PAM250; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "031");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Extension gap penalty
			if (stricmp(argv[arg], "-ex") == 0) GapEX = atof(argv[arg + 1]);

			// Opening gap penalty
			if (stricmp(argv[arg], "-op") == 0) GapOP = atof(argv[arg + 1]);

			// Minimum score allowed for matching
			if (stricmp(argv[arg], "-ms") == 0) SbScr = atof(argv[arg + 1]);

			// Keep or withdraw redundent sequences
			if (stricmp(argv[arg], "-kd") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1:	Doubles = IP_DOUBLEI; break;
					case 2:	Doubles = IP_DOUBLEO; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "032");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Output alignment format
			if (stricmp(argv[arg], "-of") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1:	OutFormat = IP_CLUSTAL; break;
					case 2:	OutFormat = IP_FASTA; break;
					case 3:	OutFormat = IP_NATIVE; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "033");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Save phylogenetic tree
			if (stricmp(argv[arg], "-st") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1: SaveTrs = true; break;
					case 2: SaveTrs = false; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "034");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Save Clusters
			if (stricmp(argv[arg], "-sc") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1:	SaveCls = true; break;
					case 2: SaveCls = false; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "035");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Save alignment
			if (stricmp(argv[arg], "-sa") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1:	SaveAln = true; break;
					case 2:	SaveAln = false; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "036");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Save alignment motifs
			if (stricmp(argv[arg], "-sm") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1:	SaveMot = true; break;
					case 2:	SaveMot = false; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "036");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Save each cluster in a separate file
			if (stricmp(argv[arg], "-sg") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1:	SaveGrps = true; break;
					case 2:	SaveGrps = false; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "036");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Perform global alignment
			if (stricmp(argv[arg], "-ga") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1:	AlignMode = IP_ALIGNALL; break;
					case 2:	AlignMode = IP_ALIGNSUB; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "037");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Perform local alignment
			if (stricmp(argv[arg], "-la") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1:	LocalAlig = IP_LOCALALY; break;
					case 2:	LocalAlig = IP_LOCALALN; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "037");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Perform iterative refinment of alignments
			if (stricmp(argv[arg], "-ir") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1:	Refinment = IP_REFWI; break;
					case 2: Refinment = IP_REFNO; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "038");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Use Names or IDs for labeling leaves
			if (stricmp(argv[arg], "-tl") == 0)
			{
				switch(atoi(argv[arg + 1]))
				{
					case 1:	Leaves = IP_LEAFN; break;
					case 2: Leaves = IP_LEAFI; break;
					default:
					{
						SendMsg("Please, enter a valid number !!!", "039");
						exit(EXIT_FAILURE);
					}
				}
			}

			// Next command
			arg += 1;
		}
	}

    // Update the substitution matrix
	UpdtSubMatrix();
}
// ---------------------------------------------------------------------------
// Save Clusters
void CSeqs::SaveClusters()
{
	if (SaveCls == true)
	{
		int i, j;
		FILE *file = fopen(OutFile, "w");

		fprintf(file, "ALIGNER clustering result \n");
		fprintf(file, "Executing time : %d seconds\n", time(NULL) - Start);
		fprintf(file, "-------------------------------------------------\n");
		fprintf(file, "-------------------------------------------------\n\n");
		fprintf(file, "Number of Clusters : %d\n", NGrps);
		fprintf(file, "-------------------------------------------------\n");

		for (i=0; i<NGrps; i++)
		{
			fprintf(file, "Cluster %d : %d sequence(s)\n\n", i+1, NSeqs[i]);

			for (j=0; j<NSeqs[i]; j++)
			{
				fprintf(file, "%d\t", Grps[i][j]);
				fprintf(file, "%s\n", Seq[Grps[i][j]].Name1);
			}

			fprintf(file, "-------------------------------------------------\n\n");
		}

		fclose(file);
	}
}
// ---------------------------------------------------------------------------
// Extract clusters
void CSeqs::GetClusters()
{
	// Allocation mmoire
	CNode **N = new CNode*[Seqs - 1];
	memset(N, NULL, (Seqs - 1)*sizeof(CNode*));

	// Classer tous les noeuds dans un vecteur
	FindNodes(N, Root);

	// Trier les noeuds par ordre dcroissant de co-similarit
	SortNodes(N, Seqs - 1);

	// Regrouper les squences
	ClusterNodes(N);

	// Librer la mmoire
	delete[] N;

	// Save the sub-groups
	SaveClusters();
}
// ---------------------------------------------------------------------------
// Save phylogenetic family tree in Phylip file format
void CSeqs::PhyAllTree(CNode *Node)
{
	// If it is a leaf
	if (Node->Size == 1)
	{
		// Open the output file for writing
		FILE *file = fopen(PhyFile, "a");

		switch(Leaves)
		{
			case IP_LEAFN:	fprintf(file, "%s:%.4f", Node->S[0]->Name2, Node->Dist); break;
			case IP_LEAFI:	fprintf(file, "%d:%.4f", Node->S[0]->Num, Node->Dist); break;
			default:
			{
				fclose(file);
				SendMsg(FATALERR, "040");
				exit(EXIT_FAILURE);
			}
		}
		// Close the output
		fclose(file);
	}
	else
	{
        // Output stream
		FILE *file;

		// Tag
		file = fopen(PhyFile, "a");
		fprintf(file, "(\n");
		fclose(file);

		// Go to the left node
		PhyAllTree(Node->L);

		// Tag
		file = fopen(PhyFile, "a");
		fprintf(file, ",\n");
		fclose(file);

		// Go to the right node
		PhyAllTree(Node->R);

		// Tag
		file = fopen(PhyFile, "a");
		fprintf(file, ")");
		fclose(file);

		if (!Node->T)
		{
			file = fopen(PhyFile, "a");
			fprintf(file, ";");
			fclose(file);
		}
		else
		{
			file = fopen(PhyFile, "a");
			fprintf(file, ":%.4f", Node->Dist);
			fclose(file);
		}
	}
}
// ---------------------------------------------------------------------------
// Save phylogenetic family tree in Phylip file format
void CSeqs::PhySubTree(CNode *Node, int Group)
{
	if (Group > 0)
	{
		// If the current cluster includes >1 sequence
		if (NSeqs[Group - 1] > 1)
		{
			// If it is a leaf
			if (Node->Size == 1)
			{
                // Open the coresponding output file
				FILE *file = fopen(FileName(Group, "phy"), "a");

				switch(Leaves)
				{
					case IP_LEAFN:  fprintf(file, "%s:%.4f", Node->S[0]->Name2, Node->Dist); break;
					case IP_LEAFI:	fprintf(file, "%d:%.4f", Node->S[0]->Num, Node->Dist); break;

					default:
					{
						fclose(file);
						SendMsg(FATALERR, "041");
						exit(EXIT_FAILURE);
					}
				}
                // Close the output file
				fclose(file);
			}
			else
			{
                // Output stream
				FILE *file;

				// Tag
				file = fopen(FileName(Group, "phy"), "a");
				fprintf(file, "(\n");
				fclose(file);

				// Go to the left node
				PhySubTree(Node->L, Group);

				// Tag
				file = fopen(FileName(Group, "phy"), "a");
				fprintf(file, ",\n");
				fclose(file);

				// Go to the right node
				PhySubTree(Node->R, Group);

				// Tag
				file = fopen(FileName(Group, "phy"), "a");
				fprintf(file, ")");
				fclose(file);

                // Test if is the root node of the subtree
				if (Node->Group > 0)
				{
					file = fopen(FileName(Group, "phy"), "a");
					fprintf(file, ";");
					fclose(file);
				}
				else
				{
					file = fopen(FileName(Group, "phy"), "a");
					fprintf(file, ":%.4f", Node->Dist);
					fclose(file);
				}
			}
		}
	}
	else
	{
		// Go to left node
		PhySubTree(Node->L, Node->L->Group);

		// Go to right node
		PhySubTree(Node->R, Node->R->Group);
	}
}
// ---------------------------------------------------------------------------
// Save the phylogenetic tree
void CSeqs::SaveTree()
{
	if (SaveTrs == true)
	{
		// Save the global phylogenetic tree
		PhyAllTree(Root);

		// save the phylogenetic subtrees
		PhySubTree(Root, Root->Group);
	}
}
// ---------------------------------------------------------------------------
// Regrouper les noeuds dans le groupe en cours
void CSeqs::MergeNodes(CNode *Node)
{
	if (Node->Size == 1)
	{
		Grps[NGrps][NSeqs[NGrps]] = Node->Num;
		NSeqs[NGrps]++;
	}
	else
	{
		MergeNodes(Node->L);
		MergeNodes(Node->R);
	}
}
// ---------------------------------------------------------------------------
// Regrouper les noeuds par scores
void CSeqs::GetGroups(CNode *Node, float ThS)
{
	if (SubGroupNode(Node, ThS))
	{
		MergeNodes(Node);
		NGrps++;
		Node->Group = NGrps;

		// Save cluster
		SaveGroup(Node);
	}
	else
	{
		GetGroups(Node->L, ThS);
		GetGroups(Node->R, ThS);
	}
}
// ---------------------------------------------------------------------------
// Regrouper les noeuds par scores
void CSeqs::ClusterNodes(CNode **Nodes)
{
	int    i;
	int    NbrExp;
	int    NbrCoss;
	int    NbrNodes = Seqs - 1;
	bool   Find;
	float *Exp;

	Exp    = AllocateF(NbrNodes);
	NbrExp = Holder(NbrNodes, Nodes, Exp);
	Find   = false;

	for (i=0; i<NbrExp; i++)
	if (Exp[i] > SEUIL)
	{
		Find = true;
		break;
	}

	switch (Find)
	{
		case true  : NbrCoss = SeparateNodes1(Nodes, NbrNodes, Exp, NbrExp);
		case false : NbrCoss = SeparateNodes2(Nodes, NbrNodes);
	}

	// Librer la mmoire
	delete[] Exp;

	NSeqs = AllocateI(Seqs);
	Grps  = AllocateI(Seqs, Seqs);

	// Regrouper les feuilles par groupe
	GetGroups(Root, Nodes[NbrCoss]->Coss);

	NSeqs = (int*)realloc(NSeqs, NGrps*sizeof(int));
	Grps  = (int**)realloc(Grps, NGrps*sizeof(int*));

	for (i=0; i<NGrps; i++)
	Grps[i] = (int*)realloc(Grps[i], NSeqs[i]*sizeof(int));

	// Save subfamilies tree
	SaveTree();
}
// ---------------------------------------------------------------------------
// Hierarchical clustering
void CSeqs::Hierarchical()
{
	int     It;			// Id's iteration
	int     i, I;		// i_index index of new cluster to amalgamate
	int     j, J;		// j_index index of new cluster to amalgamate
	float   tmp, Dist;	// used to find closest clusters to amalgamate
	CNode **L;

	// Allocation mmoire
	L = new CNode*[Seqs];

	// Initialize clusters
	for (i=0; i<Seqs; i++)
	{
		L[i]          = new CNode[1];
		L[i]->Num     = i;
		L[i]->Size    = 1;
		L[i]->Group   = 0;
		L[i]->Length  = Seq[i].Length;
		L[i]->Alig    = true;
		L[i]->Coss    = 0.0f;
		L[i]->Dist    = 0.0f;
		L[i]->Weight  = 1.0f;
		L[i]->Score   = 0.0f;
		L[i]->L       = NULL;
		L[i]->R       = NULL;
		L[i]->T       = NULL;
		L[i]->S       = new CSeq*[1];
		L[i]->S[0]    = &Seq[i];
		Seq[i].Weight = L[i]->Weight;
	}

	for (It=0; It<Seqs-1; It++)
	{
		Dist = DEF_MAX;
		for (i=0; i<Seqs; i++)
		if (L[i] != NULL)
		{
			for (j=0; j<i; j++)
			if (L[j] != NULL)
			{
				tmp = Matrix[i][j];

				if (Dist > tmp)
				{
					Dist = tmp;
					I = i;
					J = j;
				}
			}
		}

		// Cration d'un nouveau noeuds parent
		Root          = new CNode[1];
		Root->Num     = It;
		Root->Size    = L[I]->Size + L[J]->Size;
		Root->Group   = 0;
		Root->Length  = 0;
		Root->Alig    = false;
		Root->Dist    = 0.0f;
		Root->Score   = 0.0f;
		Root->Weight  = L[I]->Weight + L[J]->Weight;
		Root->S       = NULL;
		Root->T       = NULL;

		// Noeud gauche
		Root->L       = L[I];
		Root->L->T    = Root;

		// Noeud droite
		Root->R       = L[J];
		Root->R->T    = Root;

		// Distances from child to parent
		Root->L->Dist = Dist * Root->R->Weight / (Root->L->Weight + Root->R->Weight);
		Root->R->Dist = Dist * Root->L->Weight / (Root->L->Weight + Root->R->Weight);

		// Compute the co-similarity
		Root->Coss    = Dist * (Root->L->Weight * Root->R->Weight) / (Root->L->Weight + Root->R->Weight);

		// Mise  jour de la matrice de distances
		for (i=0; i<I; i++)
		Matrix[I][i] = (L[I]->Weight * Matrix[I][i] + L[J]->Weight * Matrix[J][i]) / (L[I]->Weight + L[J]->Weight);

		for (i=0; i<Seqs; i++)
		Matrix[i][I] = (L[I]->Weight * Matrix[I][i] + L[J]->Weight * Matrix[J][i]) / (L[I]->Weight + L[J]->Weight);

		L[I] = Root;
		L[J] = NULL;
	}

	// Librer la mmoire
	delete[] L;

	// Extract clusters
	GetClusters();
}
// ---------------------------------------------------------------------------
// Spectral decomposition clustering
void CSeqs::Spectral1()
{
	int     It, Dim;
	int     i, j;
	int     I, J;
	float   Sim;
	float **A;
	CNode **L;

	// Allocation mmoire
	A = new float*[Seqs];
	for (i=0; i<Seqs; i++)
	{
		A[i] = new float[i];
		memcpy(A[i], Matrix[i], i*sizeof(float));
	}

	// Spectral decomposition
	Dim = Eigen(Matrix, Seqs);

	// Initialize clusters
	L = new CNode*[Seqs];
	for (i=0; i<Seqs; i++)
	{
		L[i]          = new CNode[1];
		L[i]->Num     = i;
		L[i]->Size    = 1;
		L[i]->Group   = 0;
		L[i]->Length  = Seq[i].Length;
		L[i]->Alig    = true;
		L[i]->Coss    = 0.0f;
		L[i]->Score   = 0.0f;
		L[i]->Dist    = 0.0f;
		L[i]->Weight  = Norme(Matrix[i], Dim);
		L[i]->L       = NULL;
		L[i]->R       = NULL;
		L[i]->T       = NULL;
		L[i]->S       = new CSeq*[1];
		L[i]->S[0]    = &Seq[i];
	}

	for (It=0; It<Seqs-1; It++)
	{
		Sim = DEF_MIN;
		for (i=0; i<Seqs; i++)
		if (L[i] != NULL)
		{
			for (j=0; j<i; j++)
			if (L[j] != NULL)
			{
				if (Sim < A[i][j])
				{
					Sim = A[i][j];
					I = i;
					J = j;
				}
			}
		}

		// Create a new vector
		for (i=0; i<Dim; i++)
		Matrix[I][i] = (L[I]->Weight * Matrix[I][i] + L[J]->Weight * Matrix[J][i]) / (L[I]->Weight + L[J]->Weight);

		// Cration d'un nouveau noeuds parent
		Root          = new CNode[1];
		Root->Num     = It;
		Root->Size    = L[I]->Size + L[J]->Size;
		Root->Weight  = L[I]->Weight + L[J]->Weight;
		Root->Group   = 0;
		Root->Length  = 0;
		Root->Alig    = false;
		Root->Dist    = 0.0f;
		Root->Score   = 0.0f;
		Root->S       = NULL;
		Root->T       = NULL;

		// Noeud gauche
		Root->L       = L[I];
		Root->L->T    = Root;

		// Noeud droite
		Root->R       = L[J];
		Root->R->T    = Root;

		// Distances from child to parent
		Root->L->Dist = Root->R->Weight / (Root->L->Weight + Root->R->Weight);
		Root->R->Dist = Root->L->Weight / (Root->L->Weight + Root->R->Weight);

		// Compute the co-similarity
		Root->Coss = (1.0f - Sim) * (Root->L->Weight * Root->R->Weight) / (Root->L->Weight + Root->R->Weight);

		// Update nodes
		L[I] = Root;
		L[J] = NULL;

		for (i=0; i<I; i++)
		A[I][i] = Inner(Matrix[i], Matrix[I], Dim);

		for (i=I+1; i<Seqs; i++)
		A[i][I] = Inner(Matrix[i], Matrix[I], Dim);
	}

	// Librer la mmoire
	for (i=0; i<Seqs; i++)
	delete[] A[i];
	delete[] A;
	delete[] L;

	// Extract clusters
	GetClusters();
}
// ---------------------------------------------------------------------------
// Spectral decomposition clustering
void CSeqs::Spectral2()
{
	int     It;
	int     i, j;
	int     I, J;
	float   Sim;
	float **A;
	CNode **L;

	// Singular value decomposition
	SVD(Matrix, Seqs, Seqs);

	// Allocation mmoire
	A = new float*[Seqs];
	for (i=0; i<Seqs; i++)
	{
		A[i] = new float[i];

		for (j=0; j<i; j++)
		A[i][j] = Cosin(Matrix[i], Matrix[j], Seqs);
	}

	// Initialize clusters
	L = new CNode*[Seqs];
	for (i=0; i<Seqs; i++)
	{
		L[i]          = new CNode[1];
		L[i]->Num     = i;
		L[i]->Size    = 1;
		L[i]->Group   = 0;
		L[i]->Length  = Seq[i].Length;
		L[i]->Alig    = true;
		L[i]->Coss    = 0.0f;
		L[i]->Score   = 0.0f;
		L[i]->Dist    = 0.0f;
		L[i]->Weight  = Norme(Matrix[i], Seqs);
		L[i]->L       = NULL;
		L[i]->R       = NULL;
		L[i]->T       = NULL;
		L[i]->S       = new CSeq*[1];
		L[i]->S[0]    = &Seq[i];
	}

	for (It=0; It<Seqs-1; It++)
	{
		Sim = DEF_MIN;
		for (i=0; i<Seqs; i++)
		if (L[i] != NULL)
		{
			for (j=0; j<i; j++)
			if (L[j] != NULL)
			{
				if (Sim < A[i][j])
				{
					Sim = A[i][j];
					I   = i;
					J   = j;
				}
			}
		}

		// Create a new vector
		for (i=0; i<Seqs; i++)
		Matrix[I][i] = Matrix[I][i] + Matrix[J][i];

		// Cration d'un nouveau noeuds parent
		Root          = new CNode[1];
		Root->Num     = It;
		Root->Size    = L[I]->Size + L[J]->Size;
		Root->Group   = 0;
		Root->Length  = 0;
		Root->Alig    = false;
		Root->Dist    = 0.0f;
		Root->Score   = 0.0f;
		Root->S       = NULL;
		Root->T       = NULL;

		// Noeud gauche
		Root->L       = L[I];
		Root->L->T    = Root;

		// Noeud droite
		Root->R       = L[J];
		Root->R->T    = Root;

		// Weight of the new node
		Root->Weight  = Norme(Matrix[I], Seqs);

		// Distances from child to parent
		Root->L->Dist = Root->R->Weight / (Root->L->Weight + Root->R->Weight);
		Root->R->Dist = Root->L->Weight / (Root->L->Weight + Root->R->Weight);

		// Compute the co-similarity
		Root->Coss = (1.0f - Sim) * (Root->L->Weight * Root->R->Weight) / (Root->L->Weight + Root->R->Weight);

		// Update nodes
		L[I] = Root;
		L[J] = NULL;

		for (i=0; i<I; i++)
		A[I][i] = Cosin(Matrix[i], Matrix[I], Seqs);

		for (i=I+1; i<Seqs; i++)
		A[i][I] = Cosin(Matrix[i], Matrix[I], Seqs);
	}

	// Librer la mmoire
	for (i=0; i<Seqs; i++)
	delete[] A[i];
	delete[] A;
	delete[] L;

	// Extract clusters
	GetClusters();
}
// ---------------------------------------------------------------------------
// Regrouper les squences
void CSeqs::Clustering()
{
	// Information
	if (VERS == EXEC) printf("Cluster sequences\n");

	// phylogenetic build strategy
	switch(TASK)
	{
		case ALIGNER1: Spectral2(); break;			// ALIGNER1 alignment algorithm
		case ALIGNER2: Spectral2(); break;			// ALIGNER2 alignment algorithm
		case ALIGNER3: Spectral2(); break;			// ALIGNER3 alignment algorithm
		default:
		{
			SendMsg(FATALERR, "042");
			exit(EXIT_FAILURE);
		}
	}
}
// ---------------------------------------------------------------------------
// Remove doubles sequences
void CSeqs::FindDoubled()
{
	int   i, j, k;
	bool *Double;

	Double = AllocateB(Seqs);

	// Look for the doubles
	for (i=0; i<Seqs; i++)
	if (!Double[i])
	{
		for (j=i+1; j<Seqs; j++)
		if (!Double[j])
		{
			if (Seq[i].Length != Seq[j].Length)
				continue;

			bool find = false;

			for (k=0; k<Seq[i].Length; k++)
			if (Seq[i].Res[k] != Seq[j].Res[k])
			{
				find = true;
				break;
			}

			// This seuqnce is a double
			if (!find)
			{
				Double[j] = true;
				Doubled++;
				break;
			}
		}
	}

	// Remove doubles from the set
	if (Doubled > 0)
	{
		i = 0;
		j = 0;

		while (i < Seqs-j)
		{
			if (Double[Seq[i].Num] == true)
			{
				delete[] Seq[i].Res;

				for (k=i; k<Seqs-j-1; k++) Seq[k] = Seq[k+1];

				j++;
			}
			else i++;
		}

		// The new number of sequences
		Seqs   = i;
		SeqMax = 0;

		// Get the new longest sequence
		for (i=0; i<Seqs; i++)
		{
			// The new sequence ids
			Seq[i].Num = i;

			// If longer
			if (Seq[SeqMax].Length < Seq[i].Length) SeqMax = i;
		}
	}
}
// ---------------------------------------------------------------------------
// Load sequences from FASTA format
void CSeqs::LoadFasta()
{
	int   i;
	char  Buff[MAXLEN];
	FILE *file;

	// Verbose
	if (VERS == EXEC) printf("Load sequences from input file : %s\n", InpFile);

	// On ouvre le fichier en lecture
	if ((file = fopen(InpFile, "r")) == NULL)
	{
		SendMsg(LOADERR, "045");
		exit(EXIT_FAILURE);
	}

	// Vrifier le format du fichier FASTA
	if (!FastaFormat(file, Buff))
	{
		fclose(file);
		SendMsg(Buff, "046");
		exit(EXIT_FAILURE);
	}

	// Retourner au dbut du fichier
	fseek(file, 0, SEEK_SET);

	// Chercher le nombre de squences
	Seqs = 0;
	while (!feof(file))
	{
		// read the current line from the input file
		if (fgets(Buff, MAXLEN, file) == NULL) continue;

		// Trim special characters
		strtrim(Buff);

		// Increment the number of sequences
		if (Buff[0] == '>') Seqs++;
	}

	// Nombre de squences maximum (HTML version)
	if (VERS == HTML && Seqs > MaxAllow)
	{
		fclose(file);
		sprintf(Buff, "The server autorizes a maximum of %d sequences !!!", MaxAllow);
		SendMsg(Buff, "047");
		exit(EXIT_FAILURE);
	}

	// Retourner au dbut du fichier
	fseek(file, 0, SEEK_SET);

	// Initialiser le nombre de squences
	Seq = new CSeq[Seqs];

	// Chercher la longueur de chaque squence
	Seqs = 0;

	while (!feof(file))
	{
		// read the current line from the input file
		if (fgets(Buff, MAXLEN, file) == NULL) continue;

		// Trim special characters
		strtrim(Buff);

		if (Buff[0] == '>')
		{
			// Initialiser le vecteur de squences
			Seq[Seqs].Num    = Seqs;
			Seq[Seqs].Length = 0;
			Seq[Seqs].Weight = 0.0f;

			// Get the name of the sequence
			strcpy(Seq[Seqs].Name1, Buff + 1);
			strtok(Buff, " ");
        	strcpy(Seq[Seqs].Name2, Buff + 1);

			// Increment sequences number
			Seqs++;

			// Go to the next line
			continue;
		}

		// Increment the length of the sequence
		Seq[Seqs - 1].Length += strlen(Buff);

		// get the maximum length
		if (Seq[SeqMax].Length < Seq[Seqs - 1].Length) SeqMax = Seqs - 1;
	}
	// Intialisation des vecteur squences et frquences
	for (i=0; i<Seqs; i++)
	Seq[i].Res = AllocateC(Seq[i].Length);

	// Retourner au dbut du fichier
	fseek(file, 0, SEEK_SET);

	// Lire le contenu de chaque squence
	Seqs = 0;

	while (!feof(file))
	{
		// read the current line from the input file
		if (fgets(Buff, MAXLEN, file) == NULL) continue;

		// Trim special characters
		strtrim(Buff);

		if (Buff[0] == '>')
		{
			// Initialize the protein length
			Seq[Seqs++].Length = 0;
			continue;
		}

		for (i=0; i<strlen(Buff); i++)
		{
			if (Buff[i] == '.' || Buff[i] == '~') Buff[i] = '-';
			if (Buff[i] == '*' || Buff[i] == 'U') Buff[i] = 'X';
			if (isalpha(Buff[i]) || (Buff[i]) == '-') Seq[Seqs-1].Res[Seq[Seqs-1].Length++] = Index(toupper(Buff[i]));
		}
	}
	// Fermer le fichier
	fclose(file);

	// Information sur le fichiers
	if (VERS == EXEC)
	{
		printf("Number of sequences : %d\n", Seqs);
		printf("Longest Sequence    : %s\n", Seq[SeqMax].Name2);
		printf("Longest length      : %d\n", Seq[SeqMax].Length);

		// Enlever les doublons
		if (Doubles == IP_DOUBLEO)
		{
			FindDoubled();
			printf("Doubled sequences   : %d\n", Doubled);
		}
	}

	// Starting time
	Start = time(NULL);
}
// ---------------------------------------------------------------------------
// Expected length of the longest common motif (Karlin)
int CSeqs::Karlin(CSeq *S1, CSeq *S2)
{
	int   i;
	int   Exp;
	float Lam;
	float R1, *P1;
	float R2, *P2;

	// Allocation mmoire
	R1 = 0.0f;
	R2 = 0.0f;

	// Allocation mmoire
	P1 = AllocateF(SYMBOLS+1);

	// Allocation mmoire
	P2 = AllocateF(SYMBOLS+1);

	// Residues occurences
	for (i=0; i<S1->Length; i++)
	P1[S1->Res[i]] += 1.0f;

	// Residues occurences
	for (i=0; i<S2->Length; i++)
	P2[S2->Res[i]] += 1.0f;

	// Residues frequences
	for (i=0; i<SYMBOLS; i++)
	P1[i] /= S1->Length;

	// Residues frequences
	for (i=0; i<SYMBOLS; i++)
	P2[i] /= S2->Length;

	for (i=0; i<SYMBOLS; i++)
	R1 += pow(P1[i], 2);

	for (i=0; i<SYMBOLS; i++)
	R2 += pow(P2[i], 2);

	// Compute the matching scores
	Lam = MAX2(R1, R2);
	Exp = (log(pow((float)S1->Length, 2) + pow((float)S2->Length, 2)) + log(Lam * (1.0f - Lam)) + 0.577f) / (-log(Lam)) + (2.0f * 1.283f / fabs(log(Lam)));

	// Libration de la mmoire
	delete[] P1;
	delete[] P2;

	// return value
	return Exp;
}
// ---------------------------------------------------------------------------
// SMS matching score
void CSeqs::ExactMatch(CSeq *S1, CSeq *S2, int Min)
{
	int   i, j;
	int   k, l;
	int   Match;

	for (i=0; i<S1->Length; i++)
	{
		// Init counter
		Match = 0;

		for (k=i, j=0; k<S1->Length && j<S2->Length; k++, j++)
		{
			if (S1->Res[k] == S2->Res[j])
			{
				Match++;
			}
			else
			{
				if (Match > Min)
				{
					Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + Match)*sizeof(CRes));
					Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + Match)*sizeof(CRes));

					for (l=0; l<Match; l++)
					{
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 1;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = k-l-1;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = j-l-1;

						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 1;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = j-l-1;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = k-l-1;

						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

						Motifs[S1->Num].Nbr++;
						Motifs[S2->Num].Nbr++;
					}
				}
				// Init counter
				Match = 0;
			}
		}
		if (Match > Min)
		{
			Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + Match)*sizeof(CRes));
			Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + Match)*sizeof(CRes));

			for (l=0; l<Match; l++)
			{
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 1;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = k-l-1;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = j-l-1;

				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 1;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = j-l-1;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = k-l-1;

				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

				Motifs[S1->Num].Nbr++;
				Motifs[S2->Num].Nbr++;
			}
		}
	}

	for (i=1; i<S2->Length; i++)
	{
		// Init counter
		Match = 0;

		for (k=0, j=i; k<S1->Length && j<S2->Length; k++, j++)
		{
			if (S1->Res[k] == S2->Res[j])
			{
				Match++;
			}
			else
			{
				if (Match > Min)
				{
					Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + Match)*sizeof(CRes));
					Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + Match)*sizeof(CRes));

					for (l=0; l<Match; l++)
					{
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 1;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = k-l-1;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = j-l-1;

						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 1;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = j-l-1;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = k-l-1;

						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

						Motifs[S1->Num].Nbr++;
						Motifs[S2->Num].Nbr++;
					}
				}
				// Init counter
				Match = 0;
			}
		}
		if (Match > Min)
		{
			Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + Match)*sizeof(CRes));
			Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + Match)*sizeof(CRes));

			for (l=0; l<Match; l++)
			{
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 1;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = k-l-1;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = j-l-1;

				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 1;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = j-l-1;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = k-l-1;

				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

				Motifs[S1->Num].Nbr++;
				Motifs[S2->Num].Nbr++;
			}
		}
	}
}
// ---------------------------------------------------------------------------
// SMS matching score
void CSeqs::PermtMatch(CSeq *S1, CSeq *S2, int Min)
{
	int   i, j;
	int   k, l;
	int   Match;

	for (i=0; i<S1->Length; i++)
	{
		// Init counter
		Match = 0;

		for (k=i, j=0; k<S1->Length && j<S2->Length; k++, j++)
		{
			if (SUB_MAT1[S1->Res[k]][S2->Res[j]] > SbScr || S1->Res[k] == S2->Res[j])
			{
				Match++;
			}
			else
			{
				if (Match > Min)
				{
					Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + Match)*sizeof(CRes));
					Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + Match)*sizeof(CRes));

					for (l=0; l<Match; l++)
					{
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 1;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = k-l-1;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = j-l-1;

						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 1;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = j-l-1;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = k-l-1;

						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

						Motifs[S1->Num].Nbr++;
						Motifs[S2->Num].Nbr++;
					}
				}
				// Init counter
				Match = 0;
			}
		}
		if (Match > Min)
		{
			Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + Match)*sizeof(CRes));
			Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + Match)*sizeof(CRes));

			for (l=0; l<Match; l++)
			{
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 1;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = k-l-1;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = j-l-1;

				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 1;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = j-l-1;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = k-l-1;

				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

				Motifs[S1->Num].Nbr++;
				Motifs[S2->Num].Nbr++;
			}
		}
	}

	for (i=1; i<S2->Length; i++)
	{
		// Init counter
		Match = 0;

		for (k=0, j=i; k<S1->Length && j<S2->Length; k++, j++)
		{
			if (SUB_MAT1[S1->Res[k]][S2->Res[j]] > SbScr || S1->Res[k] == S2->Res[j])
			{
				Match++;
			}
			else
			{
				if (Match > Min)
				{
					Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + Match)*sizeof(CRes));
					Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + Match)*sizeof(CRes));

					for (l=0; l<Match; l++)
					{
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 1;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = k-l-1;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = j-l-1;

						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 1;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = j-l-1;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = k-l-1;

						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

						Motifs[S1->Num].Nbr++;
						Motifs[S2->Num].Nbr++;
					}
				}
				// Init counter
				Match = 0;
			}
		}
		if (Match > Min)
		{
			Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + Match)*sizeof(CRes));
			Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + Match)*sizeof(CRes));

			for (l=0; l<Match; l++)
			{
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 1;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = k-l-1;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = j-l-1;

				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 1;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = j-l-1;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = k-l-1;

				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

				Motifs[S1->Num].Nbr++;
				Motifs[S2->Num].Nbr++;
			}
		}
	}
}
/*
// ---------------------------------------------------------------------------
// SMS matching score
void CSeqs::PermtMatch(CSeq *S1, CSeq *S2, int Min)
 {
	int   i, j, k, l;
	int   Mach, Miss;

	for (i=0; i<S1->Length; i++)
	{
		Mach = 0;
		Miss = 0;

		for (k=i, j=0; k<S1->Length && j<S2->Length; k++, j++)
		{
			if (S1->Res[k] == S2->Res[j])
				Mach++;
			else
				Miss++;

			if (Miss >= Min || SM1[S1->Res[k]][S2->Res[j]] <= SbScr)
			{
				if (Mach > Min)
				{
					// Skip last mismatch
					Miss--;

					// Resize the vector of patterns
					Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + Mach + Miss)*sizeof(CRes));
					Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + Mach + Miss)*sizeof(CRes));

					for (l=0; l<Mach+Miss; l++)
					{
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 1;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = k-l-1;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = j-l-1;

						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 1;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = j-l-1;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = k-l-1;

						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

						Motifs[S1->Num].Nbr++;
						Motifs[S2->Num].Nbr++;
					}
				}
				// Init counters
				Mach = 0;
				Miss = 0;
			}
		}
		if (Mach > Min && Miss < Min)
		{
			// Resize the vector of patterns
			Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + Mach + Miss)*sizeof(CRes));
			Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + Mach + Miss)*sizeof(CRes));

			for (l=0; l<Mach+Miss; l++)
			{
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 1;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = k-l-1;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = j-l-1;

				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 1;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = j-l-1;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = k-l-1;

				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

				Motifs[S1->Num].Nbr++;
				Motifs[S2->Num].Nbr++;
			}
		}
	}

	for (i=1; i<S2->Length; i++)
	{
		Mach = 0;
		Miss = 0;

		for (k=0, j=i; k<S1->Length && j<S2->Length; k++, j++)
		{
			if (S1->Res[k] == S2->Res[j])
				Mach++;
			else
				Miss++;

			if (Miss >= Min || SM1[S1->Res[k]][S2->Res[j]] <= SbScr)
			{
				if (Mach > Min)
				{
					// Skip last mismatch
					Miss--;

					// Resize the vector of patterns
					Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + Mach + Miss)*sizeof(CRes));
					Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + Mach + Miss)*sizeof(CRes));

					for (l=0; l<Mach+Miss; l++)
					{
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 1;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = k-l-1;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = j-l-1;

						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 1;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = j-l-1;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = k-l-1;

						Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
						Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

						Motifs[S1->Num].Nbr++;
						Motifs[S2->Num].Nbr++;
					}
				}
				// Init counters
				Mach = 0;
				Miss = 0;
			}
		}
		if (Mach > Min && Miss < Min)
		{
			// Resize the vector of patterns
			Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + Mach + Miss)*sizeof(CRes));
			Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + Mach + Miss)*sizeof(CRes));

			for (l=0; l<Mach+Miss; l++)
			{
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 1;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = k-l-1;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = j-l-1;

				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 1;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = j-l-1;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = k-l-1;

				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

				Motifs[S1->Num].Nbr++;
				Motifs[S2->Num].Nbr++;
			}
		}
	}
}
*/
// ---------------------------------------------------------------------------
// Build the profile matrix to align a pair of groups
void CSeqs::SPatterns(CSeq *S1, CSeq *S2)
{
	switch(TASK)
	{
		case ALIGNER1: ExactMatch(S1, S2, MINDIAG); break;
		case ALIGNER2: ExactMatch(S1, S2, Karlin(S1, S2)); break;
		case ALIGNER3: PermtMatch(S1, S2, Karlin(S1, S2)); break;
		default:
		{
			SendMsg(FATALERR, "048");
			exit(EXIT_FAILURE);
		}
	}
}
// ---------------------------------------------------------------------------
// Collect significant patterns
void CSeqs::Significant()
{
	// Information sur le fichiers
	if (VERS == EXEC) printf("Detect significant patterns\n");

	// Memory allocation
	Motifs = AllocateM(Seqs);

	// Collect all residues belonging to significant patterns
	for (int i=0; i<Seqs; i++)
	for (int j=0; j<i; j++)
	{
		// Pairwise matching
		SPatterns(&Seq[i], &Seq[j]);

		// Local alignment
		LocPWAlig(&Seq[i], &Seq[j]);
	}
}
// ---------------------------------------------------------------------------
// Pairwise similarity matrix
void CSeqs::MatrixALIGNER()
{
	int    i, j;
	int    k, l;
	int    m, n;
	int    ll, rr;
	float *Scores;

	// Memory allocation
	Matrix = AllocateF(Seqs, Seqs);
	Scores = AllocateF(Seq[SeqMax].Length * Seq[SeqMax].Length);

	// Pairwise similarity
	for (i=0; i<Seqs; i++)
	for (j=i; j<Seqs; j++)
	{
		// Reset scores
		memset(Scores, 0, Seq[i].Length * Seq[j].Length * sizeof(float));

		// Consider significant patterns
		for (k=0; k<Motifs[i].Nbr; k++)
		{
			// if is not matched go next
			if (Motifs[i].Res[k].Seq2 != j) continue;

			ll = Motifs[i].Res[k].Res1;
			rr = Motifs[i].Res[k].Res2;

			// Get direct matching
			Scores[ll*Seq[j].Length + rr] += SUB_MAT1[Seq[i].Res[ll]][Seq[j].Res[rr]];
		}
		// Global alignment score
		Matrix[i][j] = Matrix[j][i] = GlbPWAlig(Scores, &Seq[i], &Seq[j]);
	}

	// Librer la mmoire
	delete[] Scores;
}
// ---------------------------------------------------------------------------
// // Weights of the sequences
void CSeqs::WeightSeq(CNode *N, float W)
{
	if (N->Size == 1)
	{
		N->S[0]->Weight = N->Dist + W;
	}
	else
	{
		WeightSeq(N->L, W + N->L->Dist/N->Size);
		WeightSeq(N->R, W + N->R->Dist/N->Size);
	}
}
// ---------------------------------------------------------------------------
// Build the spanning tree
void CSeqs::SpanningTree(CList *D, CVertx *V, int Var, int Nbr)
{
	int     i, j;
	int     I, J;
	int     T;
	CVertx *T1;
	CVertx *T2;

	// Create vertices
	for (i=0; i<Var; i++)
	{
		V[i].Num   = i;
		V[i].Group = i;
		V[i].Size  = 1;
		V[i].Next  = NULL;
	}

	// Loop until one node remain
	for (i=0, j=0; i<Nbr, j<Var-1; i++)
	{
		I = V[D[i].I].Group;
		J = V[D[i].J].Group;

		if (I == J) continue;

		// Select the current edge
		D[j++] = D[i];

		// Get nearest nodes
		if (V[I].Size > V[J].Size)
		{
			I = V[D[i].I].Group;
			J = V[D[i].J].Group;
		}
		else
		{
			I = V[D[i].J].Group;
			J = V[D[i].I].Group;
		}

		// Update vertices
		for (T1=&V[J], V[I].Size=0; T1!=NULL; T1=T1->Next, V[I].Size++)
		{
			V[T1->Num].Group = I;

			if (T1->Next != NULL)
				T2 = T1->Next;
			else
				T2 = T1;
		}
		T2->Next  = V[I].Next;
		V[I].Next = &V[J];
	}
}
// ---------------------------------------------------------------------------
// Fill the list of ngrams
void Fill(CSeq *S, int *I, int *D, int Min)
{
	int i, j, k;

	// Init the list
	memset(I, 0, D[Min]*sizeof(int));

	for (i=0; i<S->Length-Min+1; i++)
	{
		k = 0;
		for (j=0; j<Min; j++)
		k += S->Res[i + j] * D[j];

		I[k]++;
	}
}
// ---------------------------------------------------------------------------
// SMS matching score
float Quick(CSeq *S1, CSeq *S2, int *I, int *D, int Min)
{
	int   i, j, k;
	float Score = 0.0f;

	for (i=0; i<S1->Length-Min+1; i++)
	{
		k = 0;
		for (j=0; j<Min; j++)
		k += S1->Res[i + j] * D[j];

		if (I[k] == 0) continue;

		for (j=0; j<Min; j++)
		Score += SUB_MAT1[S1->Res[i + j]][S1->Res[i + j]];
	}

	return Score / MAX2(S1->Length, S2->Length);
}
// ---------------------------------------------------------------------------
// Regrouper les squences
void CSeqs::Similarity()
{
	// Information
	if (VERS == EXEC) printf("Pairwise similarity\n");

	// phylogenetic build strategy
	switch(TASK)
	{
		case ALIGNER1:	MatrixALIGNER(); break;		// ALIGNER1 alignment algorithm
		case ALIGNER2:	MatrixALIGNER(); break;		// ALIGNER2 alignment algorithm
		case ALIGNER3:	MatrixALIGNER(); break;		// ALIGNER3 alignment algorithm

		default:
		{
			SendMsg(FATALERR, "049");
			exit(EXIT_FAILURE);
		}
	}
}
// ---------------------------------------------------------------------------
// Look for if a given residue in a given sequence is significant
bool CSeqs::Significant(CNode *N, int Seq, int Res)
 {
	// Look if it is a significant residue
	for (int i=0; i<Motifs[Seq].Nbr; i++)
	{
		// If not the same residue go next
		if (Motifs[Seq].Res[i].Type != 1) continue;
		if (Motifs[Seq].Res[i].Res1 != Res) continue;
		if (Motifs[Seq].Res[i].Res2 != Res) continue;

		for (int j=0; j<N->Size; j++)
		if (Motifs[Seq].Res[i].Seq2 != N->S[j]->Num) return true;
	}

	// It is not a significant residue
	return false;
}
// ---------------------------------------------------------------------------
// Save alignment in rtf file format
void CSeqs::SaveToRTF(CNode *N)
{
	if (SaveMot == true && N->Size > 1)
	{
		int   i, j, k, m, n, l;
		int   Width, First, Last, Grap;
		bool  Found;
		float Max;
		FILE *file;

		file = fopen(FileName(N->Group, "rtf"), "w");

		// Prepare the rtf file format header
		fprintf(file, "{\\rtf1\\ansi\\ansicpg1252\\deff0\\deflang3084{\\fonttbl{\\f0\\fmodern\\fprq1\\fcharset0 Courier New;}}\n");
		fprintf(file, "{\\colortbl ;");
		fprintf(file, "\\red0\\green0\\blue0;");
		fprintf(file, "\\red0\\green0\\blue255;");
		fprintf(file, "\\red0\\green255\\blue0;");
		fprintf(file, "\\red255\\green0\\blue0;}");
		fprintf(file, "\n\\viewkind4\\uc1\\pard\\cf1\\b\\f0\\fs20 \n");

		switch(OutFormat)
		{
			case IP_CLUSTAL:	Width = 0;
								First = 0;
								Last = MIN2(LINE_CLUSTAL, N->Length);

								while (Width < N->Length)
								{
									for (i=0; i<N->Size; i++)
									{
										Grap = 0;
										fprintf(file, "\\cf1 %-40s", N->S[i]->Name2);

										for (j=First; j<Last; j++)
										{
											if ((Grap++) == 10)
											{
												Grap = 1;
												fprintf(file, "%c", ' ');
											}

											// Look if is significant
											if (Significant(N, N->S[i]->Num, j))
												fprintf(file, "\\cf4 %c", Char(N->S[i]->Res[j]));
											else
												fprintf(file, "\\cf1 %c", Char(N->S[i]->Res[j]));
										}
										fprintf(file, "\\par\n");
									}
									fprintf(file, "\\par\n");
									fprintf(file, "\\par\n");

									Width += LINE_CLUSTAL;
									First += LINE_CLUSTAL;
									Last = MIN2(Last + LINE_CLUSTAL, N->Length);
								}
								break;

			case IP_FASTA:		Width = 0;
								for (i=0; i<N->Size; i++)
								{
									Width = 0;
									fprintf(file, "\\cf1 >%s", N->S[i]->Name1);

									for (j=0; j<N->Length; j++)
									{
										// Look if is significant
										if (Significant(N, N->S[i]->Num, j) == true)
											fprintf(file, "\\cf4 %c", Char(N->S[i]->Res[j]));
										else
											fprintf(file, "\\cf1 %c", Char(N->S[i]->Res[j]));

										Width += 1;
										if (Width == LINE_FASTA)
										{
											fprintf(file, "\\par\n");
											Width = 0;
										}
									}
									if (Width != 0) fprintf(file, "\\par\n");
								}
								break;

			case IP_NATIVE:		for (i=0; i<N->Size; i++)
								{
									fprintf(file, "\\cf1 %-40s", N->S[i]->Name2);

									for (j=0; j<N->Length; j++)
									{
										if (Significant(N, N->S[i]->Num, j) == true)
											fprintf(file, "\\cf4 %c", Char(N->S[i]->Res[j]));
										else
											fprintf(file, "\\cf1 %c", Char(N->S[i]->Res[j]));
									}
									// Next line
									fprintf(file, "\\par\n");
								}
								break;
			default:
			{
				SendMsg(FATALERR, "050");
				exit(EXIT_FAILURE);
			}
		}
		// last line
		fprintf(file, "\\cf0\\par\n}\n");

		// Close file
		fclose(file);
	}
}
// ---------------------------------------------------------------------------
// Save alignment in txt file format
void CSeqs::SaveToTXT(CNode *N)
{
	if (SaveAln == true && N->Size > 1)
	{
		int   i, j, Width;
		int   First, Last, Grap;
		FILE *file;

		switch (OutFormat)
		{
			case IP_CLUSTAL:	file = fopen(FileName(N->Group, "aln"), "w");
								fprintf(file, "//\n\n");
								Width = 0;
								First = 0;
								Last = MIN2(LINE_CLUSTAL, N->Length);

								while (Width < N->Length)
								{
									for (i=0; i<N->Size; i++)
									{
										Grap = 0;
										fprintf(file, "%-40s", N->S[i]->Name2);

										for (j=First; j<Last; j++)
										{
											if ((Grap++) == 10)
											{
												Grap = 1;
												fprintf(file, "%c", ' ');
											}
											fprintf(file, "%c", Char(N->S[i]->Res[j]));
										}
										fprintf(file, "\n");
									}
									fprintf(file, "\n\n");

									Width += LINE_CLUSTAL;
									First += LINE_CLUSTAL;
									Last = MIN2(Last + LINE_CLUSTAL, N->Length);
								}
								fclose(file);
								break;

			case IP_FASTA:		file = fopen(FileName(N->Group, "aln"), "w");
								Width = 0;
								for (i=0; i<N->Size; i++)
								{
									Width = 0;
									fprintf(file, ">%s\n", N->S[i]->Name1);

									for (j=0; j<N->Length; j++)
									{
										fprintf(file, "%c", Char(N->S[i]->Res[j]));
										Width += 1;
										if (Width == LINE_FASTA)
										{
											fprintf(file, "\n");
											Width = 0;
										}
									}
									if (Width != 0) fprintf(file, "\n");
								}
								fclose(file);
								break;

			case IP_NATIVE :	file = fopen(FileName(N->Group, "aln"), "w");
								for (i=0; i<N->Size; i++)
								{
									fprintf(file, ">%-40s", N->S[i]->Name2);

									for (j=0; j<N->Length; j++)
										fprintf(file, "%c", Char(N->S[i]->Res[j]));

									fprintf(file, "\n");
								}
								fclose(file);
								break;
			default:
			{
				SendMsg(FATALERR, "051");
				exit(EXIT_FAILURE);
			}
		}
	}
}
// ---------------------------------------------------------------------------
// Epdate the alignment with the new one
void CSeqs::AlignUpdt(CNode *N1, CNode *N2)
{
    N1->Score  = N2->Score;
	N1->Length = N2->Length;

	for (int i=0; i<N1->Size; i++)
	{
		N1->S[i]->Res = (char*)realloc(N1->S[i]->Res, N2->Length*sizeof(char));

		for (int j=0; j<N2->Size; j++)
		{
			if (N1->S[i]->Num == N2->S[j]->Num)
			{
				// Clone the sequence
				memcpy(N1->S[i]->Res, N2->S[j]->Res, N2->Length * sizeof(char));

				// Go next
				break;
			}
		}
	}
}
// ---------------------------------------------------------------------------
// Insert object in a string
void CSeqs::InsertGap(CNode *N, int Pos, int Len)
{
	for (int i=0; i<N->Size; i++)
	{
		// Shift right all residues located after Pos
		for (int j=Len-1; j>Pos; j--)
		N->S[i]->Res[j] = N->S[i]->Res[j-1];

		// Insert gap at the position Pos
		N->S[i]->Res[Pos] = SYMBOLS;

		for (int j=0; j<Motifs[N->S[i]->Num].Nbr; j++)
		{
			// If current residue is located at the left of Pos go next
			if (Motifs[N->S[i]->Num].Res[j].Res1 < Pos) continue;

			// Shift residue to the right
			Motifs[N->S[i]->Num].Res[j].Res1++;

			// Shift matched residue to the left
			Motifs[Motifs[N->S[i]->Num].Res[j].Seq2].Res[Motifs[N->S[i]->Num].Res[j].Twin].Res2++;
		}
	}
}
// ---------------------------------------------------------------------------
// Align significant motifs
float CSeqs::GlbPWAlig(float *P, CSeq *S1, CSeq *S2)
{
	int    i, j;
	int    l = S1->Length;
	int    r = S2->Length;
	float  Score = 0.0f;
	float *A;
	float *H;
	float *V;

	// Allocaton mmoire
	A = AllocateF((l+1)*(r+1));
	H = AllocateF((l+1)*(r+1));
	V = AllocateF((l+1)*(r+1));

	// Init the first row and the first column
	for (i=1; i<l+1; i++)
	{
		A(i,0) = - GapOP - i * GapEX;
		H(i,0) = DEF_MIN;
		V(i,0) = - GapOP - i * GapEX;
	}

	for (i=1; i<r+1; i++)
	{
		H(0,i) = - GapOP - i * GapEX;
		V(0,i) = DEF_MIN;
		A(0,i) = - GapOP - i * GapEX;
	}

	// Smith-Waterman with affine gap costs
	for (i=1; i<l+1; i++)
	for (j=1; j<r+1; j++)
	{
		H(i,j) = MAX2(H(i,j-1) - GapEX, A(i,j-1) - GapOP - GapEX);
		V(i,j) = MAX2(V(i-1,j) - GapEX, A(i-1,j) - GapOP - GapEX);
		A(i,j) = MAX3(A(i-1,j-1) + P(i-1,j-1) + SUB_MAT1[S1->Res[i-1]][S2->Res[j-1]], H(i,j), V(i,j));
	}

	// Get the alignment score
	Score = A(l,r) / MAX2(l, r);

	// Empty memory
	delete[] H;
	delete[] V;
	delete[] A;

	// Return he alignment score
	return Score;
}
// ---------------------------------------------------------------------------
// Smith and Watternman local alignment algorithm
float CSeqs::LocPWAlig(CSeq *S1, CSeq *S2)
{
	// If do not perform local alignment return score = 0;
	if (LocalAlig == IP_LOCALALN) return 0.0f;

	int    i, j;
	int    I, J;
	int    l = S1->Length;
	int    r = S2->Length;
	float  M=0.0f;
	float  Score=0.0f;
	float *A;
	float *V;
	float *H;

	// Allocaton mmoire
	A = AllocateF((l+1)*(r+1));
	V = AllocateF((l+1)*(r+1));
	H = AllocateF((l+1)*(r+1));

	// Fill dynamic matrices
	for (i=1; i<l+1; i++)
	for (j=1; j<r+1; j++)
	{
		H(i,j) = MAX2(H(i,j-1), A(i,j-1) - GapOP);
		V(i,j) = MAX2(V(i-1,j), A(i-1,j) - GapOP);
		A(i,j) = MAX4(A(i-1,j-1) + SUB_MAT2[S1->Res[i-1]][S2->Res[j-1]], V(i,j), H(i,j), 2.0f * GapEX);

		if (M < A(i,j))
		{
			M = A(i,j);
			I = i;
			J = j;
		}
	}

	// From the maximum score position
	while (I > 0 && J > 0 && A(I,J) > 0.0f)
	{
		if (A(I,J) == H(I,J))
		{
			J--;
			Score += SUB_MAT1[SYMBOLS][S2->Res[J]];
		}
		else
		if (A(I,J) == V(I,J))
		{
			I--;
			Score += SUB_MAT1[S1->Res[I]][SYMBOLS];
		}
		else
		{
			I--;
			J--;
			Score += SUB_MAT1[S1->Res[I]][S2->Res[J]];

			if (SUB_MAT1[S1->Res[I]][S2->Res[J]] > SbScr)
			{
				// Add significant resdiue
				Motifs[S1->Num].Res = (CRes*)realloc(Motifs[S1->Num].Res, (Motifs[S1->Num].Nbr + 1)*sizeof(CRes));
				Motifs[S2->Num].Res = (CRes*)realloc(Motifs[S2->Num].Res, (Motifs[S2->Num].Nbr + 1)*sizeof(CRes));

				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Indx = Motifs[S1->Num].Nbr;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Type = 2;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq1 = S1->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res1 = I;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Seq2 = S2->Num;
				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Res2 = J;

				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Indx = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Type = 2;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq1 = S2->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res1 = J;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Seq2 = S1->Num;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Res2 = I;

				Motifs[S1->Num].Res[Motifs[S1->Num].Nbr].Twin = Motifs[S2->Num].Nbr;
				Motifs[S2->Num].Res[Motifs[S2->Num].Nbr].Twin = Motifs[S1->Num].Nbr;

				Motifs[S1->Num].Nbr++;
				Motifs[S2->Num].Nbr++;
			}
		}
	}

	// Empty memory
	delete[] A;
	delete[] H;
	delete[] V;

	// Return local alignment score
	return Score / MAX2(l,r);
}
// ---------------------------------------------------------------------------
// Needleman and Wunsch global alignment algorithm
void CSeqs::Alignment(float *P, CNode *N)
{
	int    i, j;
	int    l = N->L->Length;
	int    r = N->R->Length;
	CNode *L = N->L;
	CNode *R = N->R;
	float *OL, *PL, *UL;
	float *OR, *PR, *UR;
	float *A;
	float *H;
	float *V;

	// Allocaton mmoire
	OL = AllocateF(l);
	PL = AllocateF(l);
	UL = AllocateF(l);
	OR = AllocateF(r);
	PR = AllocateF(r);
	UR = AllocateF(r);
	A  = AllocateF((l+1)*(r+1));
	H  = AllocateF((l+1)*(r+1));
	V  = AllocateF((l+1)*(r+1));

	// Count openning gaps in left node
	for (i=1; i<l; i++)
	for (j=0; j<L->Size; j++)
	{
		if (L->S[j]->Res[i] != SYMBOLS)
		{
			UL[i] += 1.0f;
			if (L->S[j]->Res[i-1] != SYMBOLS) PL[i] += 1.0f;
		}
		else
		if (L->S[j]->Res[i-1] != SYMBOLS) OL[i] += 1.0f;
	}

	// Count openning gaps in right node
	for (i=1; i<r; i++)
	for (j=0; j<R->Size; j++)
	{
		if (R->S[j]->Res[i] != SYMBOLS)
		{
			UR[i] += 1.0f;
			if (R->S[j]->Res[i-1] != SYMBOLS) PR[i] += 1.0f;
		}
		else
		if (R->S[j]->Res[i-1] != SYMBOLS) OR[i] += 1.0f;
	}

	// Init the first row and the first column
	for (i=1; i<l+1; i++)
	{
		A(i,0) = - GapOP;
		H(i,0) = DEF_MIN;
		V(i,0) = - GapOP;
	}

	for (i=1; i<r+1; i++)
	{
		A(0,i) = - GapOP;
		H(0,i) = - GapOP;
		V(0,i) = DEF_MIN;
	}

	for (i=1; i<L->Length+1; i++)
	for (j=1; j<R->Length+1; j++)
	{
		H(i,j) = MAX2(H(i,j-1), A(i, j-1) - GapOP * (OR[j-1] + PR[j-1]) * UL[i-1] / (L->Size * R->Size));
		V(i,j) = MAX2(V(i-1,j), A(i-1, j) - GapOP * (OL[i-1] + PL[i-1]) * UR[j-1] / (L->Size * R->Size));
		A(i,j) = MAX3(A(i-1,j-1) + P(i-1,j-1) - GapOP * (OL[i-1] * UR[j-1] + OR[j-1] * UL[i-1]) / (L->Size * R->Size), H(i,j), V(i,j));
	}

	// Get the alignment score
	N->Score = A(l,r);

	// From the last row and the last column
	i = l;
	j = r;

	while (i > 0 || j > 0)
	{
		N->Length++;
		if (A(i,j) == H(i,j))
		{
			j--;
		}
		else
		if (A(i,j) == V(i,j))
		{
			i--;
		}
		else
		{
			i--;
			j--;
		}
	}

	// Resize the left node vectors
	for (i=0; i<L->Size; i++)
	L->S[i]->Res = (char*)realloc(L->S[i]->Res, N->Length*sizeof(char));

	// Resize the right node vectors
	for (i=0; i<R->Size; i++)
	R->S[i]->Res = (char*)realloc(R->S[i]->Res, N->Length*sizeof(char));

	// A partire de la dernire ligne et la dernire colonne
	i = l;
	j = r;

	while (i > 0 || j > 0)
	{
		if (A(i,j) == H(i,j))
		{
			// Shift up
			j--;

			// Insert gap in the left sequences at the position i
			InsertGap(L, i, N->Length);
		}
		else
		if (A(i,j) == V(i,j))
		{
			// Shift left
			i--;

			// Insert gap in the right sequences at the position j
			InsertGap(R, j, N->Length);
		}
		else
		{
			i--;
			j--;
		}
	}

	// Libration de la mmoire
	delete[] A;
	delete[] H;
	delete[] V;
	delete[] OL;
	delete[] PL;
	delete[] UL;
	delete[] OR;
	delete[] PR;
	delete[] UR;
}
// ---------------------------------------------------------------------------
// Align the two profiles
void CSeqs::AlignNode(CNode *N)
{
	int    i, j, k, l;
	int    ll, rr;
	float  Weight;
	CNode *L = N->L;
	CNode *R = N->R;
	float *Scores;

	// Add the left profile to the alignment result
	N->S = new CSeq*[N->Size];
	for (i=0; i<L->Size; i++)
	N->S[i] = L->S[i];

	// Add the right profile to the alignment result
	for (i=0; i<R->Size; i++)
	N->S[i + L->Size] = R->S[i];

	// Profiles matrix
	Scores = AllocateF(L->Length * R->Length);

	// Pairwize substitution scores
	for (i=0; i<L->Size; i++)
	for (j=0; j<R->Size; j++)
	{
		// Pairwise sequence weights
		Weight = (L->S[i]->Weight * R->S[j]->Weight) / (L->Size * R->Size);

		// Total Column scores
		for (k=0; k<L->Length; k++)
		for (l=0; l<R->Length; l++)
		Scores[k*R->Length + l] += SUB_MAT2[L->S[i]->Res[k]][R->S[j]->Res[l]] * Weight;

		// Get direct matched significant residues
		for (k=0; k<Motifs[L->S[i]->Num].Nbr; k++)
		{
			// Look for the significant residues in the current sequence
			if (Motifs[L->S[i]->Num].Res[k].Seq2 != R->S[j]->Num) continue;

			ll = Motifs[L->S[i]->Num].Res[k].Res1;
			rr = Motifs[L->S[i]->Num].Res[k].Res2;

			// Add the matching score to the specific position in the profile matrix
			Scores[ll*R->Length + rr] += SUB_MAT1[L->S[i]->Res[ll]][R->S[j]->Res[rr]];
		}
	}

	// PairWise alignment
	Alignment(Scores, N);

	// Node aligned
	N->Alig = true;

	// Empty memory
	delete[] Scores;
}
// ---------------------------------------------------------------------------
// Clean sequence in a node from gaped cloumns
void CSeqs::CleanNode(CNode *N)
{
	int i=0, j, k, l;
	int Gaps;

	// Withdraw gapped colomns
	while (i < N->Length)
	{
		// Init the number of gaps
		Gaps = 0;

		// Count the number of gaps in the current column
		for (j=0; j<N->Size; j++)
		if (N->S[j]->Res[i] == SYMBOLS) Gaps++;

		// Remove if the number of gaps = the number of residues in the columns
		if (Gaps < N->Size)
		{
			i++;
			continue;
		}

		// Shift left all residues
		for (k=i; k<N->Length-1; k++)
		for (l=0; l<N->Size; l++)
		N->S[l]->Res[k] = N->S[l]->Res[k+1];

		// Shift left significant residues
		for (j=0; j<N->Size; j++)
		for (k=0; k<Motifs[N->S[j]->Num].Nbr; k++)
		{
			if (Motifs[N->S[j]->Num].Res[k].Res1 <= i) continue;

			// Shift left the current residue
			Motifs[N->S[j]->Num].Res[k].Res1--;

			// Shift left the matched residue
			Motifs[Motifs[N->S[j]->Num].Res[k].Seq2].Res[Motifs[N->S[j]->Num].Res[k].Twin].Res2--;
		}
		// decrease the alignment length
		N->Length--;
	}

	// Resize Res vectors
	for (i=0; i<N->Size; i++)
	N->S[i]->Res = (char*)realloc(N->S[i]->Res, N->Length*sizeof(char));
}
// ---------------------------------------------------------------------------
// Alignment improvment
bool CSeqs::Refinement(CNode *P, CNode *N)
{
	if (N->Size > 1 && Refinment == IP_REFWI)
	{
		bool Refine = false;

		// Go left
		if (AlignImpr(P, N->L)) Refine = true;

		// Go right
		if (AlignImpr(P, N->R)) Refine = true;

		// Go deep left
		if (Refinement(P, N->L)) Refine = true;

		// Go deep right
		if (Refinement(P, N->R)) Refine = true;

		return Refine;
	}

	// No refinment
	return false;
}
// ---------------------------------------------------------------------------
// Alignment improvment
bool CSeqs::AlignImpr(CNode *T, CNode *N)
{
	int    i, j;
	bool   Found;
	bool   Refin=false;
	CSeq  *SeqsL;
	CSeq  *SeqsR;
	CNode *L;
	CNode *R;
	CNode *P;
	CMot  *MotTmp;

	MotTmp = new CMot[Seqs];
	for (i=0; i<Seqs; i++)
	{
		MotTmp[i].Nbr = Motifs[i].Nbr;
		MotTmp[i].Res = new CRes[Motifs[i].Nbr];
		memcpy(MotTmp[i].Res, Motifs[i].Res, Motifs[i].Nbr*sizeof(CRes));
	}

	// Left node contains one sequence
	L         = new CNode[1];
	L->Num    = 1;
	L->Size   = N->Size;
	L->Group  = 0;
	L->Length = T->Length;
	L->Alig   = true;
	L->Coss   = 0.0f;
	L->Score  = 0.0f;
	L->Dist   = 0.0f;
	L->Weight = 0.0f;
	L->L      = NULL;
	L->R      = NULL;
	L->T      = NULL;
	L->S      = new CSeq*[N->Size];

	// Left sequences contained
	SeqsL = new CSeq[N->Size];

	// Clone the sequences
	for (i=0; i<N->Size; i++)
	{
		// Points to the correponding sequence
		L->S[i] = &SeqsL[i];

		// Clone sequence data
		strcpy(L->S[i]->Name1, N->S[i]->Name1);
		strcpy(L->S[i]->Name2, N->S[i]->Name2);
		L->S[i]->Num    = N->S[i]->Num;
		L->S[i]->Length = N->S[i]->Length;
		L->S[i]->Weight = N->S[i]->Weight;
		L->S[i]->Res    = AllocateC(T->Length);

		// Clone residues
		memcpy(L->S[i]->Res,  N->S[i]->Res, T->Length * sizeof(char));
	}

	// Clean alignment from gaped colomns
	CleanNode(L);

	// Right node contains the remaining sequence
	R         = new CNode[1];
	R->Num    = 2;
	R->Size   = 0;
	R->Group  = 0;
	R->Length = T->Length;
	R->Alig   = true;
	R->Coss   = 0.0f;
	R->Score  = 0.0f;
	R->Dist   = 0.0f;
	R->Weight = 0.0f;
	R->L      = NULL;
	R->R      = NULL;
	R->T      = NULL;
	R->S      = new CSeq*[T->Size - N->Size];

	// Left sequences contained
	SeqsR = new CSeq[T->Size - N->Size];

	// Clone the sequences
	for (i=0; i<T->Size; i++)
	{
		Found = false;

		// Look if the sequence belong to the bipartition
		for (j=0; j<N->Size; j++)
		if (T->S[i]->Num == N->S[j]->Num)
		{
			Found = true;
			break;
		}

		// Skip if the sequence is in the bipartition
		if (Found == true) continue;

		// Points to the correponding sequence
		R->S[R->Size] = &SeqsR[R->Size];

		// Clone sequence data
		strcpy(R->S[R->Size]->Name1, T->S[i]->Name1);
		strcpy(R->S[R->Size]->Name2, T->S[i]->Name2);
		R->S[R->Size]->Num    = T->S[i]->Num;
		R->S[R->Size]->Length = T->S[i]->Length;
		R->S[R->Size]->Weight = T->S[i]->Weight;
		R->S[R->Size]->Res    = AllocateC(T->Length);

		// Clone residues
		memcpy(R->S[R->Size]->Res, T->S[i]->Res, T->Length * sizeof(char));

		// Increment the size
		R->Size++;
	}

	// Clean alignment from gaped colomns
	CleanNode(R);

	// Parent Node
	P         = new CNode[1];
	P->Num    = 3;
	P->Size   = T->Size;
	P->Group  = 0;
	P->Length = 0;
	P->Alig   = false;
	P->Coss   = 0.0f;
	P->Score  = 0.0f;
	P->Dist   = 0.0f;
	P->Weight = 0.0f;
	P->L      = L;
	P->R      = R;
	P->T      = NULL;
	P->S      = NULL;

	// Align the ith sequence with the remaining sequences
	AlignNode(P);

	// Accepte or reject refined alignment
	if (P->Score > T->Score)
	{
		// Set to refined
		Refin = true;

		// Update the alignment
		AlignUpdt(T, P);

		for (i=0; i<Seqs; i++)
		memcpy(MotTmp[i].Res, Motifs[i].Res, Motifs[i].Nbr * sizeof(CRes));
	}
	else
	{
		for (i=0; i<Seqs; i++)
		memcpy(Motifs[i].Res, MotTmp[i].Res, MotTmp[i].Nbr * sizeof(CRes));
	}

	// Empty memory
	for (i=0; i<L->Size; i++)
	delete[] SeqsL[i].Res;

	// Empty memory
	for (i=0; i<R->Size; i++)
	delete[] SeqsR[i].Res;

	for (i=0; i<Seqs; i++)
	delete[] MotTmp[i].Res;
	delete[] MotTmp;

	delete[] SeqsL;
	delete[] SeqsR;
	delete[] L->S;
	delete[] R->S;
	delete[] P->S;
	delete[] L;
	delete[] R;
	delete[] P;

	// Return if refined or no
	return Refin;
}
// ---------------------------------------------------------------------------
// Save sequence of a group
void CSeqs::SaveGroup(CNode *N)
{
	if (SaveGrps && N->Size > 1)
	{
		int   i, j;
		int   Width;
		FILE *file;

		file = fopen(FileName(N->Group, "fas"), "w");

		for (i=0; i<NSeqs[N->Group-1]; i++)
		{
			Width = 0;
			fprintf(file, ">%s\n", Seq[Grps[N->Group-1][i]].Name1);

			for (j=0; j<Seq[Grps[N->Group-1][i]].Length; j++)
			{
				fprintf(file, "%c", Char(Seq[Grps[N->Group-1][i]].Res[j]));
				Width += 1;
				if (Width == LINE_FASTA)
				{
					fprintf(file, "\n");
					Width = 0;
				}
			}
			if (Width != 0) fprintf(file, "\n");
		}
		fclose(file);
	}
}
// ---------------------------------------------------------------------------
// Alignment one subgroups
void CSeqs::AlignGrps(CNode *N)
{
	if (N->Size > 1)
	{
		if (N->L->Alig && N->R->Alig)
		{
			// Multiple alignment
			AlignNode(N);

			// If not the root go to tail
			if (N->Group == 0 && N->T != NULL) AlignGrps(N->T);
		}
		else
		{
			// Go the left child
			if (!N->L->Alig) AlignGrps(N->L);

			// Go the right child
			if (!N->R->Alig) AlignGrps(N->R);
		}
	}
}
// ---------------------------------------------------------------------------
// Regrouper les squences
void CSeqs::AlignSeqs(CNode *N)
{
	if (N->Group > 0)
	{
        // Information
		if (VERS == EXEC) printf("Align sequences in cluster #%d\n", N->Group);

		// Refinment state
		bool Refin;

		// Align a group
		AlignGrps(N);

		// Boostrap iterative refinment
		do Refin = Refinement(N, N); while (Refin);

		// Save significant motifs
		SaveToTXT(N);

		// Save alignment with significant motifs
		SaveToRTF(N);
	}
	else
	{
		AlignSeqs(N->L);
		AlignSeqs(N->R);
	}
}
// ---------------------------------------------------------------------------
// Multiple alignment
void CSeqs::Alignment()
{
	// Compute sequences' weights
	WeightSeq(Root, 0.0f);

	// Alignment of the sequences within each subfamily
	AlignSeqs(Root);

	// If perform global alignment
	if (AlignMode == IP_ALIGNALL)
	{
		// Information
		if (VERS == EXEC) printf("Align all clusters\n");

		// Refinment state
		bool Refin;

		// Alignment of the subfamilies
		AlignGrps(Root);

		// Alignment of the subfamilies
		do Refin = Refinement(Root, Root); while (Refin);

		// Save global alignment
		SaveToTXT(Root);

		// Save alignment with significant motifs
		SaveToRTF(Root);
	}
}

