//---------------------------------------------------------------------------
//
#include "Utils.hpp"
//
//---------------------------------------------------------------------------
// Singular value decomposition
void ClassicialCholesky(float **a, int n)
{
	int     i,j,k;
	float *p;
	float  sum;

    // Memory allocation
	p = AllocateF(n);

	for (i=0; i<n; i++) 
	{
		for (j=i; j<n; j++) 
		{
			sum = a[i][j];

			for (k=i-1; k>=0; k--)
			sum -= a[i][k]*a[j][k];

			if (i == j) 
			{
				if (sum < 0.0f)
				{
					printf("Cholesky decomposition failed!!!");
					exit(EXIT_FAILURE);
				}
				p[i] = sqrt(sum);
			} 
			else
			a[j][i] = sum/p[i];
		}
	}

	// Put o's on the top of the matrix
	for (i=0; i<n; i++)
	{
		a[i][i] = (float)p[i];

		for (j=0; j<i; j++)
		a[j][i] = (float)0.0f;
	}

	delete[] p;
}
//---------------------------------------------------------------------------
// Modified Cholesky factorization
void ModifiedCholesky(float **G, int n)
{
	int     i, j, k;
	float   g, zi, nu, beta;
	float  *T, *t, *E, *D;
	float **C, **L;

	g = DEF_MIN;
	for (i=0; i<n; i++)
	if (g < G[i][i]) g = G[i][i];

	zi = DEF_MIN;
	for (i=0; i<n; i++)
	for (j=0; j<i; j++)
	if (zi < G[i][j]) zi = G[i][j];

	nu = MAX2(1.0f, sqrt(pow((float)n, 2) - 1.0f));

	beta = MAX3(g, zi/nu, (float)1.0000000000E-15);

	// Allocation mmoire
	T = AllocateF(n);
	t = AllocateF(n);
	E = AllocateF(n);
	D = AllocateF(n);
	C = AllocateF(n, n);
	L = AllocateF(n, n);

	for (i=0; i<n; i++)
	{
		C[i][i] = G[i][i];

		for (j=0; j<i; j++)
		C[i][j] = C[j][i] = 0.0f;
	}

	for (j=0; j<n; j++)
	{
		if (j>0) 
		for (i=0; i<j; i++)
		L[j][i] = C[j][i] / D[i];

		if (j>=1)
		{
			if (j<n-1)
			{
				for (i=j+1; i<n; i++)
				{
					T[i] = 0.0f;

					for (k=0; k<j; k++)
					T[i] += L[i][k] * C[i][k];
				}

				for (i=j+1; i<n; i++)
				C[i][j] = G[i][j] - T[i];
			}
		}
		else
			for (i=j+1; i<n; i++)
			C[i][j] = G[i][j];

		if (j == n-1) 
			t[j] = 0.0f;
		else
		{
			t[j] = DEF_MIN;

			for (i=j+1; i<n; i++)
			if (t[j] < fabs(C[i][j])) t[j] = fabs(C[i][j]);
		}
		
		D[j] = MAX2(fabs(C[j][j]), pow(t[j], 2)/beta);

		E[j] = D[j] - C[j][j];

		for (i=j+1; i<n; i++)
		C[i][i] = C[i][i] - pow(C[i][j], 2)/D[j];
	}

	// Put 1's on the diagonal of L
	for (i=0; i<n; i++)
	L[i][i] = 1.0f;

	// Calculer les coodonnes des vecteurs
	for (i=0; i<n; i++)
	for (j=0; j<i; j++)
	if (i<=i)
		G[i][j] = L[i][j] * sqrt(D[j]);
	else
		G[i][j] = 0.0f;

	// Librer la mmoire
	for (i=0; i<n; i++)
	{
		delete[] C[i];
		delete[] L[i];
	}
	delete[] C;
	delete[] L;
	delete[] D;
	delete[] E;
	delete[] T;
	delete[] t;
}
//---------------------------------------------------------------------------
// Convert a simetric matrix to a tridiagonal matrix
void TriDiagonal(float **a, float *d, float *e, int L)
{
	int   i, j, k, l;
	float scale, hh, h, g, f;

	// Trouver les valeurs prpores
	for (i=L-1; i>=1; i--)
	{
		l = i-1;
		h = scale = 0.0f;
		if (l > 0)
		{
				for (k=0; k<=l; k++)
				scale += fabs(a[i][k]);

				if (scale == 0.0f) e[i] = a[i][l];
				else
				{
					for (k=0; k<=l; k++)
					{
						a[i][k] /= scale;
						h += a[i][k]*a[i][k];
					}

					f       = a[i][l];
					g       = (f >= 0.0f ? -sqrt(h) : sqrt(h));
					e[i]    = scale*g;
					h      -= f*g;
					a[i][l] = f-g;
					f       = 0.0f;
					for (j=0; j<=l; j++)
					{
						a[j][i] = a[i][j]/h;
						g = 0.0f;

						for (k=0; k<=j; k++) g += a[j][k]*a[i][k];
						for (k=j+1; k<=l; k++) g += a[k][j]*a[i][k];

						e[j] = g/h;
						f   += e[j]*a[i][j];
					}

					hh = f/(h+h);
					for (j=0; j<=l; j++)
					{
						f    = a[i][j];
						e[j] = g = e[j]-hh*f;
						for (k=0; k<=j; k++) a[j][k] -= (f*e[k]+g*a[i][k]);
					}
				}
		}
		else e[i] = a[i][l];

		d[i] = h;
	}

	d[0] = 0.0f;
	e[0] = 0.0f;

	// Trouver les vecteurs prpores        
	for (i=0; i<L; i++)
	{
		l = i-1;
		if (d[i])
		{
			for (j=0; j<=l; j++)
			{
				g = 0.0f;
				for (k=0; k<=l; k++) g += a[i][k]*a[k][j];
				for (k=0; k<=l; k++) a[k][j] -= g*a[k][i];
			}
		}
		d[i]    = a[i][i];
		a[i][i] = 1.0f;
		for (j=0; j<=l; j++) a[j][i] = a[i][j] = 0.0f;
	}
}
//---------------------------------------------------------------------------
// Calculate the eigen vectors and eigen values of a square matrix
int Eigen(float **a, int L)
{
	int    i, j, k, m, iter;
	int    Dim=0;
	float  s, r, p, g, f, dd, c, b;
	float *e, *d, **t;

	e = AllocateF(L);
	d = AllocateF(L);
	t = AllocateF(L, L);

	for (i=0; i<L; i++)
	memcpy(t[i], a[i], L * sizeof(float));

	// Trianguler la diaginale de la matrice
	TriDiagonal(t, d, e, L);

	for (i=1; i<L; i++) e[i-1] = e[i];

	e[L-1] = 0.0f;

	for (k=0; k<L; k++)
	{
		iter = 0;
		do
		{
				for (m=k; m<L-1; m++)
				{
						dd = fabs(d[m]) + fabs(d[m+1]);
						if ((float)(fabs(e[m])+dd) == dd) break;
				}

				if (m != k)
				{
					if (iter++ == IETRATION)
					{
						SendMsg("Too many iterations in eigenvectors searche !!!", "054");
						exit(EXIT_FAILURE);
					}
					g = (d[k+1]-d[k])/((float)2.0f*e[k]);
					r = PYTH(g,(float)1.0f);
					g = d[m]-d[k]+e[k]/(g+SIGN(r,g));
					s = c = 1.0f;
					p = 0.0f;
					for (i=m-1; i>=k; i--)
					{
						f = s*e[i];
						b = c*e[i];
						e[i+1] = (r=PYTH(f,g));
						if (r == 0.0f)
						{
								d[i+1] -= p;
								e[m] = 0.0f;
								break;
						}
						s      = f/r;
						c      = g/r;
						g      = d[i+1]-p;
						r      =(d[i]-g)*s+(float)2.0f*c*b;
						d[i+1] = g+(p=s*r);
						g      = c*r-b;

						for (j=0; j<L; j++)
						{
								f = t[j][i+1];
								t[j][i+1] = s*t[j][i]+c*f;
								t[j][i]   = c*t[j][i]-s*f;
						}
					}
					if (r==0.0f && i>=k) continue;
					d[k] -= p;
					e[k]  = g;
					e[m]  = 0.0f;
				}
		} while (m != k);
	}

	// Sort eigenvectors by eigenvalues descending order
	for (i=0; i<L; i++) e[i] = (float)i;

	for (i=0; i<L; i++)
	for (j=i+1; j<L; j++)
	if (d[i] < d[j])
	{
		s    = d[j];
		d[j] = d[i];
		d[i] = s;

		k    = (int)e[j];
		e[j] = e[i];
		e[i] = (float)k;
	}

	// Sort Eigenvectors according eigenvalues decreasing order
	for (i=0; i<L; i++)
	{
		if (d[i] <= 0.0f) break;

		Dim++;

		for (j=0; j<L; j++)
		a[j][i] = t[j][(int)e[i]];
	}

	// Calculer les coodonnes des vecteurs
	for (i=0; i<L; i++)
	for (j=0; j<Dim; j++)
	a[i][j] = a[i][j] * sqrt(d[j]);

	// Librer la mmoire
	for (i=0; i<L; i++)
	delete[] t[i];
	delete[] t;
	delete[] e;
	delete[] d;

	return Dim;
}
//---------------------------------------------------------------------------
// Singular value decomposition
void SVD(float **a, int m, int n)
{
	int    i,its,j,jj,k,l,nm;
	bool   flag;
	float  anorm,c,f,g,h,s,scale,x,y,z;
	float *w, *r, **v;

	// Allocation mmoire
	w = AllocateF(m);
	r = AllocateF(m);
	v = AllocateF(m, n);

	g = scale = anorm = 0.0f;

	for (i=0; i<n; i++)
	{
		l=i+2;
		r[i]=scale*g;
		g=s=scale=0.0f;
		if (i < m)
		{
			for (k=i;k<m;k++) scale += fabs(a[k][i]);
			if (scale != 0.0f)
			{
				for (k=i;k<m;k++)
				{
					a[k][i] /= scale;
					s += a[k][i]*a[k][i];
				}
				f=a[i][i];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][i]=f-g;
				for (j=l-1;j<n;j++)
				{
					for (s=0.0f,k=i;k<m;k++) s += a[k][i]*a[k][j];
					f=s/h;
					for (k=i;k<m;k++) a[k][j] += f*a[k][i];
				}
				for (k=i;k<m;k++) a[k][i] *= scale;
			}
		}
		w[i]=scale *g;
		g=s=scale=0.0f;
		if (i+1 <= m && i+1 != n)
		{
			for (k=l-1;k<n;k++) scale += fabs(a[i][k]);
			if (scale != 0.0f)
			{
				for (k=l-1;k<n;k++)
				{
					a[i][k] /= scale;
					s += a[i][k]*a[i][k];
				}
				f=a[i][l-1];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][l-1]=f-g;
				for (k=l-1;k<n;k++) r[k]=a[i][k]/h;
				for (j=l-1;j<m;j++)
				{
					for (s=0.0f,k=l-1;k<n;k++) s += a[j][k]*a[i][k];
					for (k=l-1;k<n;k++) a[j][k] += s*r[k];
				}
				for (k=l-1;k<n;k++) a[i][k] *= scale;
			}
		}
		anorm = MAX2(anorm,(fabs(w[i])+fabs(r[i])));
	}
	for (i=n-1;i>=0;i--)
	{
		if (i < n-1)
		{
			if (g != 0.0f)
			{
				for (j=l;j<n;j++) v[j][i]=(a[i][j]/a[i][l])/g;
				for (j=l;j<n;j++)
				{
					for (s=0.0f,k=l;k<n;k++) s += a[i][k]*v[k][j];
					for (k=l;k<n;k++) v[k][j] += s*v[k][i];
				}
			}
			for (j=l; j<n; j++) v[i][j]=v[j][i]=0.0f;
		}
		v[i][i]=1.0f;
		g=r[i];
		l=i;
	}
	for (i=MIN2(m,n)-1;i>=0;i--)
	{
		l=i+1;
		g=w[i];
		for (j=l;j<n;j++) a[i][j]=0.0f;
		if (g != 0.0f)
		{
			g=1.0f/g;
			for (j=l;j<n;j++)
			{
				for (s=0.0f,k=l;k<m;k++) s += a[k][i]*a[k][j];
				f=(s/a[i][i])*g;
				for (k=i;k<m;k++) a[k][j] += f*a[k][i];
			}
			for (j=i;j<m;j++) a[j][i] *= g;
		} else for (j=i;j<m;j++) a[j][i]=0.0f;
		++a[i][i];
	}
	for (k=n-1; k>=0; k--)
	{
		for (its=0; its<IETRATION; its++)
		{
			flag=true;
			for (l=k; l>=0; l--)
			{
				nm=l-1;
				if (fabs(r[l])+anorm == anorm)
				{
					flag=false;
					break;
				}
				if (fabs(w[nm])+anorm == anorm) break;
			}
			if (flag)
			{
				c=0.0f;
				s=1.0f;
				for (i=l;i<k+1;i++)
				{
					f=s*r[i];
					r[i]=c*r[i];
					if (fabs(f)+anorm == anorm) break;
					g=w[i];
					h=PYTH(f,g);
					w[i]=h;
					h=1.0f/h;
					c=g*h;
					s = -f*h;
					for (j=0;j<m;j++)
					{
						y=a[j][nm];
						z=a[j][i];
						a[j][nm]=y*c+z*s;
						a[j][i]=z*c-y*s;
					}
				}
			}
			z=w[k];
			if (l == k)
			{
				if (z < 0.0f)
				{
					w[k] = -z;
					for (j=0;j<n;j++) v[j][k] = -v[j][k];
				}
				break;
			}
			if (its == IETRATION-1)
			{
				SendMsg("Too many iterations in singular values decomposition !!!", "055");
				exit(EXIT_FAILURE);
			}

			x=w[l];
			nm=k-1;
			y=w[nm];
			g=r[nm];
			h=r[k];
			f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0f*h*y);
			g=PYTH(f,1.0f);
			f=((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
			c=s=1.0f;
			for (j=l;j<=nm;j++)
			{
				i=j+1;
				g=r[i];
				y=w[i];
				h=s*g;
				g=c*g;
				z=PYTH(f,h);
				r[j]=z;
				c=f/z;
				s=h/z;
				f=x*c+g*s;
				g=g*c-x*s;
				h=y*s;
				y *= c;
				for (jj=0;jj<n;jj++)
				{
					x=v[jj][j];
					z=v[jj][i];
					v[jj][j]=x*c+z*s;
					v[jj][i]=z*c-x*s;
				}
				z=PYTH(f,h);
				w[j]=z;
				if (z)
				{
					z=1.0f/z;
					c=f*z;
					s=h*z;
				}
				f=c*g+s*y;
				x=c*y-s*g;
				for (jj=0;jj<m;jj++)
				{
					y=a[jj][j];
					z=a[jj][i];
					a[jj][j]=y*c+z*s;
					a[jj][i]=z*c-y*s;
				}
			}
			r[l]=0.0f;
			r[k]=f;
			w[k]=x;
		}
	}
	delete[] r;

	// Sort singular values and vectors
	for (i=0; i<n; i++)
	for (j=i+1; j<n; j++)
	{
		if (w[i] < w[j])
		{
			SWAP(w[i], w[j]);

			for (k=0; k<m; k++) SWAP(a[k][i], a[k][j]);
			for (k=0; k<n; k++) SWAP(v[k][i], v[k][j]);
		}
	}

	// Calculer les coodonnes des vecteurs
	for (i=0; i<m; i++)
	for (j=0; j<n; j++)
	a[i][j] = v[i][j] * sqrt(w[j]);

	// Librer la mmoire
	for (i=0; i<m; i++)
	delete[] v[i];
	delete[] v;
	delete[] w;
}
